// ******************************************************************************* 
// *********
// *********      DEVICE DESCRIPTION
// *********      Target:       HCF / AMS / FC375 / PDM
// *********      DD Bus Typ:   HART
// *********      Date:         13.03.2015
// *********      Device Name:  AFIx
// *********      Label String: V1.0.0.1
// *********      Built Number: 1
// *********
// ******************************************************************************* 
//                                                                                 
// Manufactor:    Baumer                                      
//                
                                                   
//                                                               
//                Tel:
//                                                   
// Copyright:      ifak system GmbH, All Rights Reserved              
// Department:    Automation / Aut                                   
// Author:        fbi / mhu                                   
//                                                                                 
// *******************************************************************************

/*
**********************************************************************
** Manufacturer, Device type, Device revision, DD-revision
**********************************************************************
*/
MANUFACTURER 	BAUMER,
DEVICE_TYPE 	_AFIX, 

DEVICE_REVISION 1,
DD_REVISION 	1

#define CYCLE_TIME_Cond_CHART 				3000 // 0,5s update time
#define CYCLE_TIME_Conc_CHART 				3000 // 0,5s update time
#define CYCLE_TIME_TEMP_CHART 				3000 // 3s update time
#define CYCLE_TIME_CURRENT_CHART			3000 // 3s update time
#define CYCLE_TIME_Cond_CHART_2 			3000 // 0,5s update time
#define CYCLE_TIME_Conc_CHART_2 			3000 // 0,5s update time
#define CYCLE_TIME_TEMP_CHART_2 			3000 // 3s update time
#define CYCLE_TIME_CURRENT_CHART_2			3000 // 3s update time

/*
**********************************************************************
** Standard Include-files
**********************************************************************
*/

#include "macros.h"
#include "methods.h"

/*
**********************************************************************
** Device specific Include-files
**********************************************************************
*/

#include "60EC0101_dic.dd"
#include "60EC0101_var.dd"

/*
**********************************************************************
** HART Smart Communications Protocol - Common Tables
** Source Filename: Tabl20_4
**********************************************************************
*/

IMPORT STANDARD _TABLES, DEVICE_REVISION 20, DD_REVISION 4
{
	HART_DEVICE_TABLES;  // Pulls in all the items for typical HART devices.
 
   REDEFINITIONS
   {
	   
        REDEFINE VARIABLE device_type
        {
    	    LABEL device_type_lbl;
    	    HELP device_type_lbl;
           TYPE ENUMERATED (2)
            {
               { 0x60EC,  "AFI4/AFI5 CombiLyz" }
            }
        }

        VARIABLE manufacturer_id
        {
            // The type must be redefined if the Manufacturer code is not listed
            //  in the Common Tables DD
            //
	    REDEFINE LABEL manufacturer_id_lbl;
            REDEFINE TYPE ENUMERATED (2)
            {
                // Replace the 0x00f9 with your Manufacturer code and appropriate label
                { 0x0060,  "BAUMER" } 
//                { 0xAAA1,  "BAUMER" } 
            }
       }

        VARIABLE private_label_distributor
        {
            //
            //  The type must be redefined if the Private Label code is not listed
            //  in the Common Tables DD
            // 
            REDEFINE TYPE ENUMERATED (2)
            {
                // Replace the 0x00f9 with your manufactures id and appropriate label
                { 0x0060,  "BAUMER" } 
//               { 0xAAA2,  "BAUMER" } 
            }
        }
    	VARIABLE extended_fld_device_status     // see table 17
    	{
    		REDEFINE LABEL ex_device_status_lbl;
/*    		TYPE BIT_ENUMERATED
    		{     
    //			{ 0x00, no_error_lbl },
    			{ 0x01, maintenance_required_lbl },
    			{ 0x02, "Out of specification" } 
    		}
    */
    	}
        REDEFINE VARIABLE loop_current_mode_codes
        {
    		LABEL loop_current_lbl;
    		TYPE ENUMERATED 
    		{
    			{ 0, disabled_loop_current_mode_lbl },  
    			{ 1, enabled_loop_current_mode_lbl }
    		}
        }
    	REDEFINE VARIABLE loop_current_mode
    	{
    	    LABEL loop_current_lbl;
    	    TYPE ENUMERATED 
    	    {
    			{ 1, enabled_loop_current_mode_lbl },
    			{ 0, disabled_loop_current_mode_lbl }  
    	    }
    	}
    	REDEFINE VARIABLE device_status
    	{
    	    LABEL device_status_lbl;
    	    HELP [field_device_status_help];
    	    CLASS HART & DYNAMIC & DIAGNOSTIC;
    	    HANDLING READ;
    	    TYPE BIT_ENUMERATED
    	    {
    		{ 0x01, [PV_out_of_limits],             DV1 (AUTO & BAD) },
    		{ 0x02, [Non_PV_out_of_limits],         DV2 (AUTO & BAD) },
    		{ 0x04, "PV Analog Channel Saturated",  AO1 (AUTO & BAD) },
    		{ 0x08, "PV Analog Channel Fixed",      AO1 (MANUAL & GOOD) },
    		{ 0x10, [more_status_available],        MORE },
    		{ 0x20, [cold_start],                   MISC },
    		{ 0x40, [configuration_changed],        MISC },
    		{ 0x80, [device_malfunction],           MORE}
    	    }
    	}
    	REDEFINE VARIABLE write_protect
    	{
    	    LABEL [write_protect];
    	    HELP [write_protect_mode_help];
    	    CLASS DEVICE;
    	    HANDLING READ;
    	    TYPE ENUMERATED
    	    {
    		WRITE_PROTECT_CODE(0),  /* Not Write Protected */
    		WRITE_PROTECT_CODE(1),  /* Write Protected */
    		WRITE_PROTECT_CODE(251) /* None */
    	    }
    	}
        VARIABLE transfer_function_code
        {
            REDEFINE TYPE ENUMERATED
            {
        
                { 0,   linear_enum,     [linear_help] },
                { 4,   special_curve_enum,[special_curve_help] }
            }
        }
    
    }
}
/*
**********************************************************************
** HART Universal Commands
** Source Filename: univ7.ddl
**********************************************************************
*/

IMPORT STANDARD _UNIVERSAL, DEVICE_REVISION 7, DD_REVISION 2
{
    // All HART devices must support all Universal Commands
    EVERYTHING;

    REDEFINITIONS
    {
	DELETE COMMAND read_dynamic_variable_classification;		//	    NUMBER 8;
//	DELETE COMMAND read_additional_device_status;		//	    NUMBER 48;;
	DELETE VARIABLE device_specific_status_14;    
	DELETE VARIABLE device_specific_status_15;    
	DELETE VARIABLE device_specific_status_16;    
	DELETE VARIABLE device_specific_status_17;    
	DELETE VARIABLE device_specific_status_18;    
	DELETE VARIABLE device_specific_status_19;    
	DELETE VARIABLE device_specific_status_20;    
	DELETE VARIABLE device_specific_status_21;    
	DELETE VARIABLE device_specific_status_22;    
	DELETE VARIABLE device_specific_status_23;    
	DELETE VARIABLE device_specific_status_24;    
      // This must be redefined if the Manufacturer is not listed in the Common Tables DD


    COMMAND read_additional_device_status       //    NUMBER 48;
    {
        REDEFINE TRANSACTION 0
        {
            REQUEST {}
            REPLY
            {
                response_code, device_status, 
                device_specific_status_0, device_specific_status_1, device_specific_status_2,
                device_specific_status_3, device_specific_status_4, device_specific_status_5,
    
                extended_fld_device_status, 0x00,
    
                standardized_status_0, standardized_status_1, analog_channel_saturated1,
                standardized_status_2, standardized_status_3, analog_channel_fixed1
            }
        }
        REDEFINE TRANSACTION 1
        {
            REQUEST
            {
                device_specific_status_0 (INFO), device_specific_status_1 (INFO), device_specific_status_2 (INFO),
                device_specific_status_3 (INFO), device_specific_status_4 (INFO), device_specific_status_5 (INFO),
    
                extended_fld_device_status (INFO), 0x00,
    
                standardized_status_0 (INFO), standardized_status_1 (INFO), analog_channel_saturated1 (INFO),
                standardized_status_2 (INFO), standardized_status_3 (INFO), analog_channel_fixed1 (INFO)
            }
            REPLY
            {
                response_code, device_status, 
                device_specific_status_0 (INFO), device_specific_status_1 (INFO), device_specific_status_2 (INFO),
                device_specific_status_3 (INFO), device_specific_status_4 (INFO), device_specific_status_5 (INFO),
    
                extended_fld_device_status (INFO), 0x00,
    
                standardized_status_0 (INFO), standardized_status_1 (INFO), analog_channel_saturated1 (INFO),
                standardized_status_2 (INFO), standardized_status_3 (INFO), analog_channel_fixed1 (INFO)
            }
        }
    }

    VARIABLE polling_address
      {
        REDEFINE LABEL polling_address_lbl;
        REDEFINE TYPE UNSIGNED_INTEGER
         {
            MIN_VALUE 0;
            MAX_VALUE 63;
         }
      }
      
      VARIABLE tag
      {
         REDEFINE LABEL tag_lbl;
      }

      VARIABLE longTag
      {
         REDEFINE LABEL long_tag_lbl;
      }


	  VARIABLE software_revision
    	{
    	    REDEFINE LABEL software_revision_lbl;
    	}
    	
    	VARIABLE hardware_revision
    	{
    	    REDEFINE LABEL hardware_revision_lbl;
    	}
    
    	VARIABLE device_id
    	{
    		REDEFINE LABEL device_identification_lbl;
    	}

        REDEFINE VARIABLE time_stamp 
        {    
            LABEL "Time Stamp";
            HELP "Relative time value on network";
            CLASS DEVICE & DYNAMIC;
            HANDLING READ;
            TYPE TIME_VALUE;
        }
            
        COMMAND read_device_variables_and_status        //    NUMBER 9;
        {
            REDEFINE TRANSACTION
            {
                REQUEST
                {
                    device_variable_code_1 (INFO,INDEX),
                    device_variable_code_2 (INFO,INDEX),
                    device_variable_code_3 (INFO,INDEX),
                    device_variable_code_4 (INFO,INDEX)
                }
                REPLY
                {
                    response_code, device_status,
                    extended_fld_device_status,
        
                    device_variable_code_1 (INFO,INDEX),
                    deviceVariables[device_variable_code_1].CLASSIFICATION,
                    deviceVariables[device_variable_code_1].DIGITAL_UNITS,
                    deviceVariables[device_variable_code_1].DIGITAL_VALUE,
                    deviceVariables[device_variable_code_1].DATA_QUALITY <0xC0>,
                    deviceVariables[device_variable_code_1].LIMIT_STATUS <0x30>,
                    deviceVariables[device_variable_code_1].DEVICE_FAMILY_STATUS <0x0F>,
        
                    device_variable_code_2 (INFO,INDEX),
                    deviceVariables[device_variable_code_2].CLASSIFICATION,
                    deviceVariables[device_variable_code_2].DIGITAL_UNITS,
                    deviceVariables[device_variable_code_2].DIGITAL_VALUE,
                    deviceVariables[device_variable_code_2].DATA_QUALITY <0xC0>,
                    deviceVariables[device_variable_code_2].LIMIT_STATUS <0x30>,
                    deviceVariables[device_variable_code_2].DEVICE_FAMILY_STATUS <0x0F>,
        
                    device_variable_code_3 (INFO,INDEX),
                    deviceVariables[device_variable_code_3].CLASSIFICATION,
                    deviceVariables[device_variable_code_3].DIGITAL_UNITS,
                    deviceVariables[device_variable_code_3].DIGITAL_VALUE,
                    deviceVariables[device_variable_code_3].DATA_QUALITY <0xC0>,
                    deviceVariables[device_variable_code_3].LIMIT_STATUS <0x30>,
                    deviceVariables[device_variable_code_3].DEVICE_FAMILY_STATUS <0x0F>,
        
                    device_variable_code_4 (INFO,INDEX),
                    deviceVariables[device_variable_code_4].CLASSIFICATION,
                    deviceVariables[device_variable_code_4].DIGITAL_UNITS,
                    deviceVariables[device_variable_code_4].DIGITAL_VALUE,
                    deviceVariables[device_variable_code_4].DATA_QUALITY <0xC0>,
                    deviceVariables[device_variable_code_4].LIMIT_STATUS <0x30>,
                    deviceVariables[device_variable_code_4].DEVICE_FAMILY_STATUS <0x0F>,

                    time_stamp
                }
            }
        }

    }
}

/*
**********************************************************************
** HART Standard Collections
** Source Filename: PV1_1.ddl
**********************************************************************
*/
IMPORT STANDARD _PV, DEVICE_REVISION 1, DD_REVISION 1
{
    EVERYTHING;
    REDEFINITIONS
    {
        /* 
         * Most devices use the same data item for range units (command 15) and the sensor units.
         * when this is true the following should be added.  if your device uses a different variable
         * for range units you should reference it here 
         */

    
	REDEFINE VARIABLE transfer_function
	{
	    LABEL transfer_function_lbl;
	    HELP transfer_function_lbl;
	    HANDLING READ & WRITE;
	    TYPE ENUMERATED
	    {
		TRANSFER_FUNCTION_CODE (0),     // Linear 
		//TRANSFER_FUNCTION_CODE (1),     // Square Root 
		TRANSFER_FUNCTION_CODE (4)      // Spezial
	    }
	}
/*
        ARRAY OF COLLECTION dynamic_variables
        {
            REDEFINE ELEMENTS
            {
                PRIMARY,        primary_variable,       "";
                SECONDARY,      secondary_variable,     "";
            }
        }
	*/
        COLLECTION OF VARIABLE scaling
        {
            MEMBERS
            {
                ADD RANGE_UNITS, deviceVariables[0].DIGITAL_UNITS, [range_units], [range_units_help];
 		REDEFINE UPPER_RANGE_VALUE,      upperRange_value_2,   pv_urv_lbl,                  [upper_range_value_help];
		REDEFINE LOWER_RANGE_VALUE,      lowerRange_value_2,   pv_lrv_lbl,                  [lower_range_value_help];
           } 
        }
        COLLECTION OF VARIABLE analog_io
        {
            MEMBERS
            {
                /* Analog Information */
                REDEFINE ANALOG_VALUE,           loopCurrent,        loop_current_lbl,         [analog_value_help];
		REDEFINE ANALOG_CHANNEL_FLAGS,   loop_flags,         [analog_channel_flags], [analog_channel_flags_help];
            }
        }
   }
}
/*
**********************************************************************
** HART Common Practice
** Source Filename: Common9.ddl
**********************************************************************
*/

IMPORT STANDARD _COMMON_PRACTICE, DEVICE_REVISION 9, DD_REVISION 2
{
	
    COMMAND write_pv_damping_value;                     // Command 34
    //COMMAND write_pv_range_values;                      // Command 35
    //COMMAND set_pv_upper_range_value;                   // Command 36
    //COMMAND set_pv_lower_range_value;                   // Command 37
    //COMMAND enter_exit_fixed_pv_current_mode;           // Command 40
    COMMAND perform_device_reset;  		                // Command 42
    //COMMAND set_pv_zero;                   		        // Command 43
    //COMMAND write_pv_units;                   		    // Command 44
    //COMMAND trim_pv_current_dac_zero;                   // Command 45
    //COMMAND trim_pv_current_dac_gain;                   // Command 46
    COMMAND write_pv_transfer_function;		    	    // Command 47
    //COMMAND read_device_variable_assignments;           // Command 50
    COMMAND write_device_variable_units;			//NUMBER 53
//    COMMAND read_device_variable_trim_point;		    // Command 80
//    COMMAND read_device_variable_trim_guidelines;	    // Command 81
  //  COMMAND write_device_variable_trim_point;           // Command 82
    //COMMAND reset_device_variable_trim;           	    // Command 83
	

    METHOD return_to_normal;
    METHOD warning_message;
    METHOD device_self_test;
    METHOD device_master_reset;
    METHOD transmitter_loop_test;
    METHOD leave_fixed_current_mode;
    METHOD transmitter_dac_trim;
    METHOD makeItSquawk;
    METHOD trim_warning;
//    METHOD device_variable_trim_reset;

    VARIABLE device_variable_code;
    VARIABLE device_variable_trim_code;
    VARIABLE primary_variable_code;
    VARIABLE secondary_variable_code;
    VARIABLE tertiary_variable_code;
    VARIABLE quaternary_variable_code;


}

/*
 * Dynamic Variables Configuration
 */
#include "60EC0101_PV.dd"
#include "60EC0101_SV.dd"
#include "60EC0101_TV.dd"
#include "60EC0101_QV.dd"

#include "60EC0101_tab_1.dd"
#include "60EC0101_tab_2.dd"
#include "60EC0101_tab_3.dd"
#include "60EC0101_tab_4.dd"
#include "60EC0101_graph.dd"

#include "60EC0101_unitrelation.dd"


ARRAY OF COLLECTION deviceVariables 
{
    ELEMENTS
    {
        0, pv_variable_collection, " ";		//"Cond";
        1, sv_variable_collection, " ";		//"Temp";
        2, tv_variable_collection, " ";		//"Conc";
        3, qv_variable_collection, " ";		//"test";
        8, av_variable_collection, " ";		//"mA Cond/Conc";
        7, bv_variable_collection, " ";		//"mA Temp";
    }
}

ARRAY OF COLLECTION trimDeviceVariables 
{
    ELEMENTS
    {
        0, pv_variable_collection, "Conductivity";
    }
}

/*
***************************************************************************
** below is list of menu declaritions
** they where take directly from menu.ddl an interoperable file
** this is a recommended display format derived by the HART communicator
** developers in conjunction with slave developers and end users.
*/

#include "60EC0101_menu.dd"
#include "60EC0101_mth.dd"
#include "60EC0101_cmd.dd"

